CON
  _clkmode = xtal1 + pll16x               
  _xinfreq = 5_000_000
                      
 
  BTN3         = 6
  BTN4         = 8
  rangeFinder  = 14
  piezoBuzzer  = 15               
   
OBJ        'the conduit-object is for debugging with the EZLog.EXE from www.hannoware.com
  _vp    : "Conduit"                        'transfers data to/from PC
 
VAR
  long mSec

  long Btn3State
  long Btn4State
  
  long Btn3Cnt
  long Btn4Cnt
 

  long StateMachineStack[50]

          
PUB Main
  'tell EZLog the variable names
  'the variablenames should ALWAYS be the same as in the var section
  'to keep things clear
  _vp.config(string("var:Btn3Cnt,Btn4Cnt,S_Width")) 

  'share variables Btn3Cnt..S_Width with EZLog
  '@variablename means HUB-RAM adress of variable 
  _vp.share(@Btn3Cnt,@Btn4Cnt)                 

  mSec := ClkFreq / 1000
    
 
  
  'start method that checks for button-pushes in a new cog
  'the GLOBAL variable S_Width can be accessed from both cogs
  cognew(ButtonStateMachine,@StateMachineStack)

 


CON 'these constants belong to PUB ButtonStateMachine
  MinOnTime  = 100
  MinOFFTime = 200
  
  LowLimit  =  800
  HighLimit = 2200

  sm_Btn3_Wait_for_Pin_High       = 10
  sm_Btn3_Check_Pin_staying_High  = 20
  sm_Btn3_Wait_for_Pin_Low        = 30
  sm_Btn3_Check_Pin_staying_Low   = 40
  
  sm_Btn4_Wait_for_Pin_High       = 10
  sm_Btn4_Check_Pin_staying_High  = 20
  sm_Btn4_Wait_for_Pin_Low        = 30
  sm_Btn4_Check_Pin_staying_Low   = 40

  'attention ! my PPDB has pullUP-resistors which means
  'with an UNpressedbutton the inputpin has the logic state 1
  BtnPressed  = 0
  BtnReleased = 1

  {with pull-DOWN-resistor simply reverse the logic
  BtnPressed  = 1
  BtnReleased = 0
  }
  

PUB ButtonStateMachine | BCnt3, BCnt4
{{
This method does check two buttons all the time in a special way:
1: debouncing the two buttons
2. if pressed long enough increment/decrement variable S_Width
3. waiting until button is released for a minimum time
   before a new buttonpress is accepted to inc/dec the variable again 

the method contains two identical statemachines for Btn3 and Btn4
}}

  BCnt3 := 0
  BCnt4 := 0
  
  Btn3State := sm_Btn3_Wait_for_Pin_High
  Btn4State := sm_Btn4_Wait_for_Pin_High

  repeat
    WaitCnt(ClkFreq / 1000 + cnt) 'slow down repeat-loop to msecs

    'statemachine for Btn3
    case Btn3State
    
      sm_Btn3_Wait_for_Pin_High:
        if ina[BTN3] == BtnPressed
          Btn3State := sm_Btn3_Check_Pin_staying_High


          
      sm_Btn3_Check_Pin_staying_High:
        if ina[BTN3] == BtnPressed
          BCnt3++          
         'if button hasn't bounced var BCnt3 is greater than MinTime 
          if BCnt3 > MinOnTime
            BCnt3 := 0  
                                                                   
            if ina[rangeFinder] == 1                              
             repeat                                                 '\\\\\\\\\\\\\\\\\\\\\\\   NEW CODE\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
              dira[piezoBuzzer] := 1                                '*Turns piezo indicator on, pitch determined by PWM output of rangeFinder 
              waitcnt(500)                                          '*Turns buzzer into beeps so that a change in pitch is easier to detect by ear 
              dira[piezoBuzzer] := -1
              waitcnt(500)                                          '\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
            
            Btn3Cnt++
            Btn3State := sm_Btn3_Check_Pin_staying_Low

         'Button HAS bounced so start a new buttonpress-sequence 
        else  
          BCnt3 := 0
          Btn3State := sm_Btn3_Wait_for_Pin_High




      sm_Btn3_Check_Pin_staying_Low:
      
        if ina[BTN3] == BtnReleased
          BCnt3++
          
        if BCnt3 > MinOFFTime
          BCnt3 := 0  
          Btn3State := sm_Btn3_Wait_for_Pin_High
          

    
    'statemachine for Btn4
    case Btn4State
    
      sm_Btn4_Wait_for_Pin_High:
        if ina[Btn4] == BtnPressed
          Btn4State := sm_Btn4_Check_Pin_staying_High


          
      sm_Btn4_Check_Pin_staying_High:
        if ina[BTN4] == BtnPressed
          BCnt4++          
         'if button hasn't bounced var BCnt4 is greater than MinTime 
          if BCnt4 > MinOnTime
            BCnt4 := 0  
                                                                     '\\\\\\\\\\\\\\\\\\\\\\\   NEW CODE\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\  
            dira[piezoBuzzer] := -1                                  '*Turns piezo indicator off 
                                                                     '\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\  
            Btn4Cnt++
            Btn4State := sm_Btn4_Check_Pin_staying_Low

         'Button HAS bounced so start a new buttonpress-sequence 
        else  
          BCnt4 := 0
          Btn4State := sm_Btn4_Wait_for_Pin_High





      sm_Btn4_Check_Pin_staying_Low:
      
        if ina[Btn4] == BtnReleased
          BCnt4++
          
        if BCnt4 > MinOFFTime
          BCnt4 := 0  
          Btn4State := sm_Btn4_Wait_for_Pin_High
          

          